@@ -71,6 +71,8 @@ pub(crate) struct MetricFormatter<'a> {
     val: MetricValue,
     type_: MetricType,
     tags: Vec<(Option<&'a str>, &'a str)>,
+    base_size: usize,
+    kv_size: usize,
 }
 
 impl<'a> MetricFormatter<'a> {
@@ -104,6 +106,7 @@ impl<'a> MetricFormatter<'a> {
         Self::from_val(prefix, key, val, MetricType::Set)
     }
 
+    #[rustfmt::skip]
     fn from_val(prefix: &'a str, key: &'a str, val: MetricValue, type_: MetricType) -> Self {
         MetricFormatter {
             prefix,
@@ -111,15 +114,24 @@ impl<'a> MetricFormatter<'a> {
             type_,
             val,
             tags: Vec::new(),
+            // keep track of the number of bytes we expect to use for both the key-value
+            // part of the tags for this metric as well as the base metric (name, value,
+            // and type). incrementing these counters when tags are added saves us from
+            // having to loop through the tags to count the expected number of bytes to
+            // allocate.
+            kv_size: 0,
+            base_size: prefix.len() + key.len() + 1 /* : */ + 10 /* value */ + 1 /* | */ + 2, /* type */
         }
     }
 
     fn with_tag(&mut self, key: &'a str, value: &'a str) {
         self.tags.push((Some(key), value));
+        self.kv_size += key.len() + 1 /* : */ + value.len();
     }
 
     fn with_tag_value(&mut self, value: &'a str) {
         self.tags.push((None, value));
+        self.kv_size += value.len();
     }
 
     fn write_base_metric(&self, out: &mut String) {
@@ -142,39 +154,17 @@ impl<'a> MetricFormatter<'a> {
         }
     }
 
-    // Don't have rustfmt do anything to this method because it keeps wrapping
-    // the line when we don't want it to.
-    #[rustfmt::skip]
-    fn base_metric_size_hint(&self) -> usize {
-        // Justification for "10" bytes: the max number of digits we could possibly
-        // need for the string representation of our value is 20 (for both u64::MAX
-        // and i64::MIN including the minus sign). So, 10 digits covers a pretty
-        // large range of values that will actually be seen in practice. Plus, using
-        // a constant is faster than computing the `val.log(10)` of our value which
-        // we would need to know exactly how many digits it takes up.
-        self.prefix.len() + self.key.len() + 1 /* : */ + 10 /* value */ + 1 /* | */ + 2 /* type */
-    }
-
     fn tag_size_hint(&self) -> usize {
         if self.tags.is_empty() {
             return 0;
         }
 
-        let kv_size: usize = self
-            .tags
-            .iter()
-            .map(|(key, val)| {
-                // keys are optional so either include its length and ':' or zero
-                key.map(|s| s.len() + 1 /* : */).unwrap_or(0) + val.len()
-            })
-            .sum();
-
         // prefix, keys and values, commas
-        Self::TAG_PREFIX.len() + kv_size + self.tags.len() - 1
+        Self::TAG_PREFIX.len() + self.kv_size + self.tags.len() - 1
     }
 
     pub(crate) fn format(&self) -> String {
-        let size_hint = self.base_metric_size_hint() + self.tag_size_hint();
+        let size_hint = self.base_size + self.tag_size_hint();
         let mut metric_string = String::with_capacity(size_hint);
         self.write_base_metric(&mut metric_string);
         self.write_tags(&mut metric_string);
@@ -417,7 +407,13 @@ where
 
 #[cfg(test)]
 mod tests {
-    use super::{MetricFormatter, MetricValue};
+    use super::{MetricBuilder, MetricFormatter, MetricValue};
+    use crate::client::StatsdClient;
+    use crate::sinks::NopMetricSink;
+    use crate::test::ErrorMetricSink;
+    use crate::types::Counter;
+    use std::sync::atomic::{AtomicU64, Ordering};
+    use std::sync::Arc;
 
     #[test]
     fn test_metric_formatter_tag_size_hint_no_tags() {
@@ -550,4 +546,58 @@ mod tests {
             &fmt.format()
         );
     }
+
+    #[test]
+    fn test_metric_builder_send_success() {
+        let fmt = MetricFormatter::counter("prefix.", "some.counter", MetricValue::Signed(11));
+        let client = StatsdClient::builder("prefix.", NopMetricSink)
+            .with_error_handler(|e| {
+                panic!("unexpected error sending metric: {}", e);
+            })
+            .build();
+
+        // if the send failed the test would have called the error handler and panicked
+        let builder: MetricBuilder<'_, '_, Counter> = MetricBuilder::from_fmt(fmt, &client);
+        builder.send();
+    }
+
+    #[test]
+    fn test_metric_builder_send_error() {
+        let errors = Arc::new(AtomicU64::new(0));
+        let errors_ref = errors.clone();
+
+        let fmt = MetricFormatter::counter("prefix.", "some.counter", MetricValue::Signed(11));
+        let client = StatsdClient::builder("prefix.", ErrorMetricSink::always())
+            .with_error_handler(move |_e| {
+                errors_ref.fetch_add(1, Ordering::Release);
+            })
+            .build();
+
+        let builder: MetricBuilder<'_, '_, Counter> = MetricBuilder::from_fmt(fmt, &client);
+        builder.send();
+
+        assert_eq!(1, errors.load(Ordering::Acquire));
+    }
+
+    #[test]
+    fn test_metric_builder_try_send_success() {
+        let fmt = MetricFormatter::counter("prefix.", "some.counter", MetricValue::Signed(11));
+        let client = StatsdClient::from_sink("prefix.", NopMetricSink);
+
+        let builder: MetricBuilder<'_, '_, Counter> = MetricBuilder::from_fmt(fmt, &client);
+        let res = builder.try_send();
+
+        assert!(res.is_ok(), "expected Ok result from try_send");
+    }
+
+    #[test]
+    fn test_metric_builder_try_send_error() {
+        let fmt = MetricFormatter::counter("prefix.", "some.counter", MetricValue::Signed(11));
+        let client = StatsdClient::from_sink("prefix.", ErrorMetricSink::always());
+
+        let builder: MetricBuilder<'_, '_, Counter> = MetricBuilder::from_fmt(fmt, &client);
+        let res = builder.try_send();
+
+        assert!(res.is_err(), "expected Err result from try_send");
+    }
 }
